@@ -7,6 +7,6 @@ my $path = $r->args || '';
 $server = $r->construct_server;
 
 print "Location: http://$server$path\n\n";
-warn "Location: http://$server$path\n\n";
+#warn "Location: http://$server$path\n\n";
 
 __END__
