@@ -1,5 +1,14 @@
 #include "mod_perl.h"
 
+/* make sure that mod_perl won't try to start itself, while it's
+ * already starting. If the flag's value is 1 * it's still starting,
+ * when it's 2 it is running */
+static int MP_init_status = 0;
+
+#define MP_IS_NOT_RUNNING (MP_init_status == 0 ? 1 : 0)
+#define MP_IS_STARTING    (MP_init_status == 1 ? 1 : 0)
+#define MP_IS_RUNNING     (MP_init_status == 2 ? 1 : 0)
+
 #ifndef USE_ITHREADS
 static apr_status_t modperl_shutdown(void *data)
 {
@@ -101,6 +110,28 @@ apr_pool_t *modperl_server_pool(void)
     return server_pool;
 }
 
+static void set_taint_var(PerlInterpreter *perl)
+{
+    dTHXa(perl);
+    GV *gv;
+
+/* 5.7.3+ has a built-in special ${^TAINT}, backport it to 5.6.0+ */
+#if PERL_REVISION == 5 && \
+    (PERL_VERSION == 6 || (PERL_VERSION == 7 && PERL_SUBVERSION < 3))
+
+    gv = gv_fetchpv("\024AINT", GV_ADDMULTI, SVt_IV);
+    sv_setiv(GvSV(gv), PL_tainting);
+    SvREADONLY_on(GvSV(gv));
+#endif /* perl v < 5.7.3 */
+
+#ifdef MP_COMPAT_1X
+    gv = gv_fetchpv("Apache::__T", GV_ADDMULTI, SVt_PV);
+    sv_setiv(GvSV(gv), PL_tainting);
+    SvREADONLY_on(GvSV(gv));
+#endif /* MP_COMPAT_1X */
+    
+}
+
 PerlInterpreter *modperl_startup(server_rec *s, apr_pool_t *p)
 {
     AV *endav;
@@ -149,10 +180,24 @@ PerlInterpreter *modperl_startup(server_rec *s, apr_pool_t *p)
     endav = PL_endav;
     PL_endav = Nullav;
 
+/* This was fixed in 5.9.0/5.8.1 (17775), but won't compile after 19122 */
+#if PERL_REVISION == 5 && PERL_VERSION == 8 && PERL_SUBVERSION == 0 && \
+    defined(USE_REENTRANT_API) && defined(HAS_CRYPT_R) && defined(__GLIBC__)
+    /* workaround perl5.8.0/glibc bug */
+    PL_reentrant_buffer->_crypt_struct.current_saltbits = 0;
+#endif
+    
     perl_run(perl);
 
+#ifdef USE_ITHREADS
+    /* base server / virtual host w/ +Parent gets its own mip */
+    modperl_interp_init(s, p, perl);
+#endif
+
     PL_endav = endav;
 
+    set_taint_var(perl);
+    
     MP_TRACE_i(MP_FUNC, "constructed interpreter=0x%lx\n",
                (unsigned long)perl);
 
@@ -162,6 +207,13 @@ PerlInterpreter *modperl_startup(server_rec *s, apr_pool_t *p)
     );
 #endif
 
+#ifdef MP_COMPAT_1X    
+    av_push(GvAV(PL_incgv),
+            newSVpv(ap_server_root_relative(p, ""), 0));
+    av_push(GvAV(PL_incgv),
+            newSVpv(ap_server_root_relative(p, "lib/perl"), 0));
+#endif /* MP_COMPAT_1X */
+    
     if (!modperl_config_apply_PerlRequire(s, scfg, perl, p)) {
         exit(1);
     }
@@ -175,7 +227,7 @@ PerlInterpreter *modperl_startup(server_rec *s, apr_pool_t *p)
     apr_pool_cleanup_register(server_pool, cdata,
                               modperl_shutdown, apr_pool_cleanup_null);
 #endif
-    
+
     return perl;
 }
 
@@ -238,6 +290,13 @@ int modperl_init_vhost(server_rec *s, apr_pool_t *p,
                    modperl_server_desc(s, p));
     }
     else {
+#ifdef USE_ITHREADS
+        /* virtual host w/ +Clone gets its own mip */
+        if (MpSrvCLONE(scfg)) {
+            modperl_interp_init(s, p, perl);
+        }   
+#endif
+        
         if (!modperl_config_apply_PerlModule(s, scfg, perl, p)) {
             return HTTP_INTERNAL_SERVER_ERROR;
         }
@@ -247,21 +306,11 @@ int modperl_init_vhost(server_rec *s, apr_pool_t *p,
     }
 
 #ifdef USE_ITHREADS
-
-    /* if alloc flags is On or clone flag is On,
-     *  virtual host gets its own mip
-     */
-    if (MpSrvPARENT(scfg) || MpSrvCLONE(scfg)) {
-        MP_TRACE_i(MP_FUNC, "modperl_interp_init() server=%s\n",
-                   modperl_server_desc(s, p));
-        modperl_interp_init(s, p, perl);
-    }
-
     /* if we allocated a parent perl, mark it to be destroyed */
     if (MpSrvPARENT(scfg)) {
         MpInterpBASE_On(scfg->mip->parent);
     }
-
+    
     if (!scfg->mip) {
         /* since mips are created after merge_server_configs()
          * need to point to the base mip here if this vhost
@@ -282,20 +331,6 @@ void modperl_init(server_rec *base_server, apr_pool_t *p)
     modperl_config_srv_t *base_scfg;
     PerlInterpreter *base_perl;
 
-    /* get the real base server when invoked from vhost.
-     *
-     * without doing it segfaults when the first PerlLoadModule
-     * appears inside vhost, e.g.:
-     *     <VirtualHost _default_:8535>
-     *         PerlLoadModule Foo
-     *     </VirtualHost> 
-     * an arrangement which is unfortunately hard to automate with our
-     * test suite, but see test TestDirective::perlloadmodule6
-     */
-    if (base_server->is_virtual) {
-        base_server = modperl_global_get_server_rec();
-    }
-
     base_scfg = modperl_config_srv_get(base_server);
 
     MP_TRACE_d_do(MpSrv_dump_flags(base_scfg,
@@ -316,11 +351,13 @@ void modperl_init(server_rec *base_server, apr_pool_t *p)
 
     base_perl = modperl_startup(base_server, p);
 
+    MP_init_status = 2; /* only now mp has really started */
+
 #ifdef USE_ITHREADS
-    modperl_interp_init(base_server, p, base_perl);
+    /* a parent perl was allocated, mark it to be destroyed */
     MpInterpBASE_On(base_scfg->mip->parent);
 #endif
-
+    
     for (s=base_server->next; s; s=s->next) {
         if (modperl_init_vhost(s, p, base_server) != OK) {
             exit(1); /*XXX*/
@@ -416,11 +453,9 @@ static apr_status_t modperl_sys_init(void)
     return APR_SUCCESS;
 }
 
-static int MP_init_done = 0;
-
 static apr_status_t modperl_sys_term(void *data)
 {
-    MP_init_done = 0;
+    MP_init_status = 0;
 
     modperl_env_unload();
 
@@ -435,10 +470,12 @@ static apr_status_t modperl_sys_term(void *data)
 int modperl_hook_init(apr_pool_t *pconf, apr_pool_t *plog, 
                       apr_pool_t *ptemp, server_rec *s)
 {
-    if (MP_init_done++ > 0) {
+    if (MP_IS_STARTING || MP_IS_RUNNING) {
         return OK;
     }
 
+    MP_init_status = 1; /* now starting */
+
     apr_pool_create(&server_pool, pconf);
 
     modperl_sys_init();
@@ -454,19 +491,17 @@ int modperl_hook_init(apr_pool_t *pconf, apr_pool_t *plog,
  * if we need to init earlier than post_config,
  * e.g. <Perl> sections or directive handlers.
  */
-/*
- * XXX: this probably won't work well if called from a
- * vhost rather than the base config if modperl_hook_init
- * hasn't been run first from the base config.
- */
-int modperl_run(apr_pool_t *p, server_rec *s)
+int modperl_run(void)
 {
-    return modperl_hook_init(p, NULL, NULL, s);
+    return modperl_hook_init(modperl_global_get_pconf(),
+                             NULL,
+                             NULL,
+                             modperl_global_get_server_rec());
 }
 
 int modperl_is_running(void)
 {
-    return MP_init_done;
+    return MP_IS_RUNNING;
 }
 
 int modperl_hook_pre_config(apr_pool_t *p, apr_pool_t *plog,
@@ -481,8 +516,8 @@ int modperl_hook_pre_config(apr_pool_t *p, apr_pool_t *plog,
 
 static int modperl_hook_pre_connection(conn_rec *c, void *csd)
 {
-    modperl_input_filter_register_connection(c);
-    modperl_output_filter_register_connection(c);
+    modperl_input_filter_add_connection(c);
+    modperl_output_filter_add_connection(c);
     return OK;
 }
 
@@ -512,6 +547,13 @@ static int modperl_hook_create_request(request_rec *r)
 
     modperl_config_req_init(r, rcfg);
 
+    /* set the default for cgi header parsing On as early as possible
+     * so $r->content_type in any phase after header_parser could turn
+     * it off. wb->header_parse will be set to 1 only if this flag
+     * wasn't turned off and MpDirPARSE_HEADERS is on
+     */
+    MpReqPARSE_HEADERS_On(rcfg);
+    
     return OK;
 }
 
@@ -524,7 +566,7 @@ static int modperl_hook_post_read_request(request_rec *r)
 }
 
 static int modperl_hook_header_parser(request_rec *r)
-{
+{    
     /* if 'PerlOptions +GlobalRequest' is inside a container */
     modperl_global_request_cfg_set(r);
 
@@ -541,7 +583,10 @@ int modperl_perl_destruct_level(void)
 static apr_status_t modperl_child_exit(void *data)
 {
     char *level = NULL;
-
+    server_rec *s = (server_rec *)data;
+    
+    modperl_callback_process(MP_CHILD_EXIT_HANDLER, server_pool, s);
+    
     if ((level = getenv("PERL_DESTRUCT_LEVEL"))) {
         modperl_destruct_level = atoi(level);
     }
@@ -563,7 +608,7 @@ static void modperl_hook_child_init(apr_pool_t *p, server_rec *s)
 {
     modperl_perl_init_ids_server(s);
 
-    apr_pool_cleanup_register(p, NULL, modperl_child_exit,
+    apr_pool_cleanup_register(p, (void *)s, modperl_child_exit,
                               apr_pool_cleanup_null);
 }
 
@@ -595,10 +640,10 @@ void modperl_register_hooks(apr_pool_t *p)
     ap_hook_handler(modperl_response_handler_cgi,
                     NULL, NULL, APR_HOOK_MIDDLE);
 
-    ap_hook_insert_filter(modperl_output_filter_register_request,
+    ap_hook_insert_filter(modperl_output_filter_add_request,
                           NULL, NULL, APR_HOOK_LAST);
 
-    ap_hook_insert_filter(modperl_input_filter_register_request,
+    ap_hook_insert_filter(modperl_input_filter_add_request,
                           NULL, NULL, APR_HOOK_LAST);
 
     ap_register_output_filter(MP_FILTER_REQUEST_OUTPUT_NAME,
@@ -647,7 +692,12 @@ static const command_rec modperl_cmds[] = {
     MP_CMD_SRV_TAKE1("PerlPassEnv", pass_env, "PerlPassEnv"),
     MP_CMD_SRV_RAW_ARGS_ON_READ("<Perl", perl, "Perl Code"),
     MP_CMD_SRV_RAW_ARGS("Perl", perldo, "Perl Code"),
-	
+
+    MP_CMD_DIR_TAKE1("PerlSetInputFilter", set_input_filter,
+                     "filter[;filter]"),
+    MP_CMD_DIR_TAKE1("PerlSetOutputFilter", set_output_filter,
+                     "filter[;filter]"),
+    
     MP_CMD_DIR_RAW_ARGS_ON_READ("=pod", pod, "Start of POD"),
     MP_CMD_DIR_RAW_ARGS_ON_READ("=back", pod, "End of =over"),
     MP_CMD_DIR_RAW_ARGS_ON_READ("=cut", pod_cut, "End of POD"),
@@ -705,16 +755,17 @@ void modperl_response_init(request_rec *r)
     wb->pool = r->pool;
     wb->filters = &r->output_filters;
     wb->outcnt = 0;
-    wb->header_parse = MpDirPARSE_HEADERS(dcfg) ? 1 : 0;
+    wb->header_parse = MpDirPARSE_HEADERS(dcfg) && MpReqPARSE_HEADERS(rcfg)
+        ? 1 : 0;
     wb->r = r;
 }
 
-void modperl_response_finish(request_rec *r)
+apr_status_t modperl_response_finish(request_rec *r)
 {
     MP_dRCFG;
 
     /* flush output buffer */
-    modperl_wbucket_flush(rcfg->wbucket);
+    return modperl_wbucket_flush(rcfg->wbucket, FALSE);
 }
 
 static int modperl_response_handler_run(request_rec *r, int finish)
@@ -730,7 +781,10 @@ static int modperl_response_handler_run(request_rec *r, int finish)
     }
 
     if (finish) {
-        modperl_response_finish(r);
+        apr_status_t rc = modperl_response_finish(r);
+        if (rc != APR_SUCCESS) {
+            retval = rc;
+        }
     }
 
     return retval;
@@ -749,7 +803,7 @@ int modperl_response_handler_cgi(request_rec *r)
 {
     MP_dDCFG;
     GV *h_stdin, *h_stdout;
-    int retval;
+    apr_status_t retval, rc;
     MP_dRCFG;
 #ifdef USE_ITHREADS
     pTHX;
@@ -803,7 +857,10 @@ int modperl_response_handler_cgi(request_rec *r)
 #endif
 
     /* flush output buffer after interpreter is putback */
-    modperl_response_finish(r);
+    rc = modperl_response_finish(r);
+    if (rc != APR_SUCCESS) {
+        retval = rc;
+    }
 
     switch (rcfg->status) {
       case HTTP_MOVED_TEMPORARILY:
