@@ -19,6 +19,7 @@
 #include "http_vhost.h"
 #include "ap_mpm.h"
 
+#include "apr_version.h"
 #ifndef APR_POLLIN
 /*
  * apr_poll.h introduced around 2.0.40
@@ -31,26 +32,11 @@
 #include "apr_uri.h"
 #include "apr_date.h"
 #include "apr_buckets.h"
+#include "apr_time.h"
+#include "apr_network_io.h"
+
 #include "util_filter.h"
 
 #include "util_script.h"
 
-#if !APR_HAS_THREADS
-typedef unsigned long apr_os_thread_t;
-typedef void * apr_thread_mutex_t;
-#endif
-
-/* XXX: these backcompat macros can be deleted when we bump up the
- * minimal supported httpd version to 2.0.40 or higher
- */
-#ifndef apr_time_sec
-#define apr_time_sec(time) ((apr_int64_t)((time) / APR_USEC_PER_SEC))
-#endif
-#ifndef apr_time_usec
-#define apr_time_usec(time) ((apr_int32_t)((time) % APR_USEC_PER_SEC))
-#endif
-#ifndef apr_time_from_sec
-#define apr_time_from_sec(sec) ((apr_time_t)(sec) * APR_USEC_PER_SEC)
-#endif 
-
 #endif /* MODPERL_APACHE_INCLUDES_H */
