@@ -1,18 +1,5 @@
 #include "mod_perl.h"
 
-/*
- * bleedperl change #11639 switch tied handle magic
- * from living in the gv to the GvIOp(gv), so we have to deal
- * with both to support 5.6.x
- */
-#if ((PERL_REVISION == 5) && (PERL_VERSION >= 7))
-#   define TIEHANDLE_SV(handle) (SV*)GvIOp((SV*)handle)
-#else
-#   define TIEHANDLE_SV(handle) (SV*)handle
-#endif
-
-#define dHANDLE(name) GV *handle = gv_fetchpv(name, TRUE, SVt_PVIO)
-
 #define TIEHANDLE(handle,r) \
 modperl_io_handle_tie(aTHX_ handle, "Apache::RequestRec", (void *)r)
 
