@@ -3,9 +3,13 @@ package TestAPI::response;
 use strict;
 use warnings FATAL => 'all';
 
+use Apache::RequestRec ();
 use Apache::Response ();
+
 use Apache::Test;
 
+use Apache::Const -compile => 'OK';
+
 sub handler {
     my $r = shift;
 
