@@ -16,7 +16,7 @@ use Apache::Constants qw(OK DIR_MAGIC_TYPE);
 sub handler {
     my $r = shift;
 
-    plan $r, tests => 7;
+    plan $r, tests => 8;
 
     $r->send_http_header('text/plain');
 
@@ -34,6 +34,10 @@ sub handler {
              Apache->define('MODPERL2'),
              'Apache->define');
 
+    ok t_cmp('PerlResponseHandler',
+             Apache::current_callback(),
+             'inside PerlResponseHandler');
+
     Apache::log_error("Apache::log_error test ok");
     ok 1;
 
@@ -51,3 +55,6 @@ sub handler {
 
 1;
 
+__END__
+# so we can test whether send_httpd_header() works fine
+PerlOptions +ParseHeaders
