@@ -14,6 +14,8 @@ modperl_newSVsv_obj(aTHX_ stashsv, sv)
 #define mpxs_Apache__RequestRec_PRINTF mpxs_ap_rprintf
 #define mpxs_Apache__RequestRec_BINMODE(r) \
     r ? SVYES : SVNO /* noop */
+#define mpxs_Apache__RequestRec_CLOSE(r) \
+    r ? SVYES : SVNO /* noop */
 
 #define mpxs_Apache__RequestRec_UNTIE(r, refcnt) \
     (r && refcnt) ? SVYES : SVNO /* noop */
@@ -21,8 +23,7 @@ modperl_newSVsv_obj(aTHX_ stashsv, sv)
 #define mpxs_output_flush(r, rcfg) \
     /* if ($|) */ \
     if (IoFLUSH(PL_defoutgv)) { \
-        modperl_wbucket_flush(rcfg->wbucket); \
-        ap_rflush(r); \
+        MP_FAILURE_CROAK(modperl_wbucket_flush(rcfg->wbucket, TRUE)); \
     }
 
 static MP_INLINE apr_size_t mpxs_ap_rvputs(pTHX_ I32 items,
@@ -39,6 +40,7 @@ static MP_INLINE apr_size_t mpxs_ap_rvputs(pTHX_ I32 items,
 
     MP_START_TIMES();
 
+    MP_CHECK_WBUCKET_INIT("$r->puts");
     mpxs_write_loop(modperl_wbucket_write, rcfg->wbucket);
 
     MP_END_TIMES();
@@ -65,7 +67,8 @@ apr_size_t mpxs_Apache__RequestRec_print(pTHX_ I32 items,
     mpxs_usage_va_1(r, "$r->print(...)");
     
     rcfg = modperl_config_req_get(r);
-    
+
+    MP_CHECK_WBUCKET_INIT("$r->print");
     mpxs_write_loop(modperl_wbucket_write, rcfg->wbucket);
     
     mpxs_output_flush(r, rcfg);
@@ -92,7 +95,9 @@ apr_size_t mpxs_ap_rprintf(pTHX_ I32 items, SV **MARK, SV **SP)
     modperl_perl_do_sprintf(aTHX_ sv, items, MARK);
     bytes = SvCUR(sv);
 
-    modperl_wbucket_write(rcfg->wbucket, SvPVX(sv), &bytes);
+    MP_CHECK_WBUCKET_INIT("$r->printf");
+    MP_FAILURE_CROAK(modperl_wbucket_write(aTHX_ rcfg->wbucket,
+                                           SvPVX(sv), &bytes));
     
     mpxs_output_flush(r, rcfg);
 
@@ -124,11 +129,31 @@ apr_size_t mpxs_Apache__RequestRec_write(pTHX_ request_rec *r,
         wlen = bufsiz;
     }
 
-    modperl_wbucket_write(rcfg->wbucket, buf+offset, &wlen);
-
+    MP_CHECK_WBUCKET_INIT("$r->write");
+    MP_FAILURE_CROAK(modperl_wbucket_write(aTHX_ rcfg->wbucket,
+                                           buf+offset, &wlen));
+    
     return wlen;
 }
 
+static MP_INLINE
+int mpxs_Apache__RequestRec_rflush(pTHX_ I32 items,
+                                   SV **MARK, SV **SP)
+{
+    modperl_config_req_t *rcfg;
+    request_rec *r;
+    
+    /* this also magically assings to r ;-) */
+    mpxs_usage_va_1(r, "$r->rflush()");
+
+    rcfg = modperl_config_req_get(r);
+
+    MP_CHECK_WBUCKET_INIT("$r->rflush");
+    MP_FAILURE_CROAK(modperl_wbucket_flush(rcfg->wbucket, TRUE));
+
+    return APR_SUCCESS;
+}
+
 static MP_INLINE long mpxs_ap_get_client_block(pTHX_ request_rec *r,
                                                SV *buffer, int bufsiz)
 {
@@ -236,6 +261,35 @@ SV *mpxs_Apache__RequestRec_GETC(pTHX_ request_rec *r)
     return &PL_sv_undef;
 }
 
+static MP_INLINE
+int mpxs_Apache__RequestRec_OPEN(pTHX_ SV *self,  SV *arg1, SV *arg2)
+{
+    char *name;
+    STRLEN len;
+    SV *arg;
+    dHANDLE("STDOUT");
+    
+    modperl_io_handle_untie(aTHX_ handle); /* untie *STDOUT */
+ 
+    if (arg2 && self) {
+        arg = newSVsv(arg1);
+        sv_catsv(arg, arg2);
+    }
+    else {
+        arg = arg1;
+    }
+
+    name = SvPV(arg, len);
+    return do_open(handle, name, len, FALSE, O_RDONLY, 0, Nullfp);
+}
+
+static MP_INLINE
+int mpxs_Apache__RequestRec_FILENO(pTHX_ request_rec *r)
+{
+    dHANDLE("STDOUT");
+    return PerlIO_fileno(IoOFP(TIEHANDLE_SV(handle)));
+}
+
 static MP_INLINE
 apr_status_t mpxs_Apache__RequestRec_sendfile(request_rec *r,
                                               const char *filename,
