@@ -85,18 +85,8 @@ request_rec *mpxs_Apache__RequestRec_new(SV *classname,
 static MP_INLINE
 request_rec *mpxs_Apache_request(pTHX_ SV *classname, SV *svr)
 {
-    request_rec *cur = NULL;
-    apr_status_t status = modperl_tls_get_request_rec(&cur);
-
-    if (status != APR_SUCCESS) {
-        /* XXX: croak */
-    }
-
-    if (svr) {
-        modperl_global_request_obj_set(aTHX_ svr);
-    }
-
-    return cur;
+    /* ignore classname */
+    return modperl_global_request(aTHX_ svr);
 }
 
 static MP_INLINE
