@@ -1,6 +1,3 @@
-/*
- * XXX: should do something useful if we end up with any bodytext
- */
 /* XXX: this should probably named $r->cgi_header_parse
  * and send_cgi_header an alias in Apache::compat
  */
@@ -9,21 +6,13 @@
     MP_dRCFG; \
     STRLEN len; \
     const char *bodytext; \
+    MP_CGI_HEADER_PARSER_OFF(rcfg); \
     modperl_cgi_header_parse(r, SvPV(sv,len), &bodytext); \
-    rcfg->wbucket->header_parse = 0; \
-}
-
-/* XXX: should only be part of Apache::compat */
-static MP_INLINE void
-mpxs_Apache__RequestRec_send_http_header(request_rec *r, const char *type)
-{
-    MP_dRCFG;
-
-    if (type) {
-        ap_set_content_type(r, apr_pstrdup(r->pool, type));
-    }
-
-    rcfg->wbucket->header_parse = 0; /* turn off PerlOptions +ParseHeaders */
+    if (bodytext) {\
+        MP_CHECK_WBUCKET_INIT("$r->send_cgi_header"); \
+        len -= (bodytext - SvPVX(sv)); \
+        modperl_wbucket_write(aTHX_ rcfg->wbucket, bodytext, &len); \
+    } \
 }
 
 static MP_INLINE void
